#!/bin/usr/sage 
"""
This is program written in sage for rounding exact solutions from flag algebra computations.
Does not do everything, but can do something if you are lucky.
In general, you need to provide
 - SDP program  dat-s and its solution dat-s.result from CSDP
 - eigenvectors corresponding to zero eigenvalues of matrices in the solution.

NOTE: SDP program needs to have ALL entries integers. So before rounding, you need
to multiply the objective function or constraints accordingly. This is because the
rounding is done over integers to avoid rounding errors.

The rounding procedure is trying to guess graphs, where the CSDP program
must be tight. Usually, subgraphs of the extremal example. You may provide them
or hope the program can guess them right.

Few more notes:
 - solutions can be seen as matrices but also as one long vector
   during rounding, we look at the solution as a long vector.
 

# TODO:
# print graph ID togehter with densities from primal and dual
# to identify the tight ones
#   - add also ones from blow-up
#   - try to push away those not tight with objective function
"""

import numpy
import subprocess
import sys



field=Integer
################################################################################### UTILITIES
# Reads the whole problem as integer program
# the program has to be integral. This is a hack
# for SDP programs that were solved without scaled objective
# and are expensive to resolve
def read_problem_file(filename,constants_rescale=1,constraint_from=0, constraint_to=10000000):
    f = open(filename, 'r')

    line = f.readline()
    line = line.strip()
    num_constraints = int(line)

    line = f.readline()
    line = line.strip()
    num_blocks = int(line)

    line = f.readline()
    parts = line.split()
    block_sizes = []
    for part in parts:
        block_sizes.append(int(part))

    line = f.readline()
    parts = line.split()

    constants = []
    used_constraints=0
    if constants_rescale == 1:
        for i in xrange(num_constraints):
           if i >= constraint_from and i < constraint_to:
               constants.append(field(parts[i]))
               used_constraints += 1
    else:
        for i in xrange(num_constraints):
           if i >= constraint_from and i < constraint_to:
               constants.append(field(int(float(parts[i])*constants_rescale+0.5)))
               used_constraints += 1

    # blocks that are integer numbers right away
    blocks = []
    for size in block_sizes:
        if size < 0:
            #block = [ matrix(ZZ,1,-size) for x in range(num_constraints+1) ]
            block = [ matrix(ZZ,1,-size) for x in range(used_constraints+1) ]
        else:
            block = [ matrix(ZZ,size,size, sparse=True) for x in range(used_constraints+1) ]
            #block = [ matrix(ZZ,size,size, sparse=True) for x in range(num_constraints+1) ]
            #block = [ matrix(QQ,size,size) for x in range(num_constraints+1) ]
        blocks.append(block)


    while True:
        line = f.readline()
        if len(line) == 0:
            break
        parts = line.split()
        if len(parts) == 0:
            continue
        constraint = int(parts[0])
        # Read also a bit arround to accommodate for mistakes..
        # Blocks are shifted by one
        #if constraint <= constraint_from or constraint > constraint_to:
        #    continue
        block_num = int(parts[1])
        row = int(parts[2])
        column = int(parts[3])
        if constants_rescale == 1:
            value = field(parts[4])
        else:
            value = field(float(parts[4])*constants_rescale)

        if block_sizes[block_num - 1] < 0:
            blocks[block_num - 1][constraint-constraint_from][0, column - 1] += value
        else:
            blocks[block_num - 1][constraint-constraint_from][row - 1, column - 1] += value
            if row != column:
                blocks[block_num - 1][constraint-constraint_from][column - 1, row - 1] += value

    problem = dict()
    problem['num_constraints'] = used_constraints
    problem['num_blocks'] = num_blocks
    problem['block_sizes'] = block_sizes
    problem['blocks'] = blocks
    problem['constants'] = constants
    f.close()
    return problem


def write_problem_file(filename, problem):
    f = open(filename, 'w')

    num_constraints = problem['num_constraints']
    num_blocks = problem['num_blocks']
    block_sizes = problem['block_sizes']
    blocks = problem['blocks']
    constants = problem['constants']

    f.write("%d\n" % num_constraints)
    f.write("%d\n" % num_blocks)
    for size in block_sizes:
        f.write("%d " % size)
    f.write("\n")

    for constant in constants:
        f.write("%.13g " % constant)
    f.write("\n")

    def print_row():
        if value != 0:
            f.write("%d %d %d %d %.13g\n" % (constraint, block_num + 1, row + 1, column + 1, value))

    for constraint in xrange(num_constraints + 1):
        for block_num in xrange(num_blocks):
            block_size = block_sizes[block_num]
            if block_size < 0:
                for column in xrange(-block_size):
                    row = column
                    #print 'Writing',block_num,column,constraint
                    value = blocks[block_num][constraint][0,column]
                    print_row()
            else:
                for row in xrange(block_size):
                    for column in xrange(row, block_size):
                        value = blocks[block_num][constraint][row,column]
                        print_row()

    f.close()



# This reads the solution file - but only the primal. Dual is being ignored
# and it would waste space anyway. The problem is read as real numbers - because
# we 
def read_solution_file(filename, problem):
    f = open(filename, 'r')

    num_constraints = problem['num_constraints']
    num_blocks = problem['num_blocks']
    block_sizes = problem['block_sizes']

    Y = numpy.zeros([num_constraints], float)
    line = f.readline()
    parts = line.split()
    for i in xrange(num_constraints):
        Y[i] = float(parts[i])

    Primal = []
    Dual = []


    # ignore Dual ############################ We work only with primal anyway

    for block_num in xrange(num_blocks):
        if block_sizes[block_num] < 0:
            Primal.append(matrix(RDF, 1, -block_sizes[block_num]))
            #Dual.append(numpy.zeros([1, -block_sizes[block_num]], float))
        else:
            Primal.append(matrix(RDF, block_sizes[block_num], block_sizes[block_num]))
            #Dual.append(numpy.zeros([block_sizes[block_num], block_sizes[block_num]], float))

    while True:
        line = f.readline()
        if len(line) == 0:
            break
        parts = line.split()
        if len(parts) == 0:
            continue
        constraint = int(parts[0])
        block_num = int(parts[1])
        row = int(parts[2])
        column = int(parts[3])
        value = float(parts[4])

        if constraint == 2:
            block = Primal[block_num - 1]
        else:
            continue
            #block = Dual[block_num - 1]

        if block_sizes[block_num - 1] < 0:
            block[0, column - 1] += value
        else:
            block[row - 1, column - 1] += value
            if row != column:
                block[column - 1, row - 1] += value


    solution = dict()
    solution['Y'] = Y
    solution['Primal'] = Primal
    solution['Dual'] = Dual
    return solution



def checkSolution(problem, variables):
   """Evaluates problem (program) with variables. It works as a test if you want to see if your
   solution is any good.
   """
   num_constraints = problem['num_constraints']
   num_blocks = problem['num_blocks']
   block_sizes = problem['block_sizes']
   blocks = problem['blocks']
   constants = problem['constants']
   RoundingLinearSystemVariableNumber = len(variables)
   OriginalProbelmMatrix = matrix(QQ, num_constraints, RoundingLinearSystemVariableNumber)
   for i in range(num_constraints):
      k = 0
      for index in range(num_blocks):
          size = block_sizes[index]
          if size > 0:
             for x in range(size):
                for y in range(x,size):
                   if x == y:
                      OriginalProbelmMatrix[i, k] = blocks[index][i+1][x,y]
                   else:
                      OriginalProbelmMatrix[i, k] = 2 * blocks[index][i+1][x,y]
                   k += 1
          if size < 0 and -size != num_constraints+1:
              for x in range(-size):
                 OriginalProbelmMatrix[i, k] = blocks[index][i+1][0,x]
                 k += 1
   constantsVector = vector(QQ,constants)
   constantsVector = vector(QQ,constantsVector)
   densityVector = constantsVector - OriginalProbelmMatrix * variables
   return densityVector

# Function to write Ping's solution 
def printLinear(linear,denominator):
    sys.stdout.write('denominatorLinear='+str(denominator)+'\n')
    sys.stdout.write('solutionLinear=[')
    for i in range(len(linear)):
        if i != 0:
            sys.stdout.write(', ')
        sys.stdout.write(str(linear[i]*denominator))
    sys.stdout.write(']')
    sys.stdout.write('\n]')

def printMatrix(roundedMatrices,denominator):
    sys.stdout.write('denominatorLinear='+str(denominator)+'\n')
    sys.stdout.write('solutionLinear=[')
    for b in range(len(roundedMatrices)):
        if b != 0:
            sys.stdout.write(',\n')
        for i in range(len(roundedMatrices[b][0])):
            for j in range(i,len(roundedMatrices[b][0])):
                if i != 0 or j != 0: 
				   sys.stdout.write(', ')
                sys.stdout.write(str(roundedMatrices[b][i,j]*denominator))
    sys.stdout.write(']')
    sys.stdout.write('\n]')



def linearToMatrix(blocks,dim,linearSolution):
   M=[]
   cnt = 0
   for b in range(blocks):
       if dim[b] > 0:
           MS = MatrixSpace(QQ,dim[b])
           M.append(MS())
           for i in range(dim[b]):
               for j in range(i,dim[b]):
                   M[b][j,i] = M[b][i,j] = linearSolution[cnt]
                   cnt=cnt+1
       if dim[b] < 0:
           MS = MatrixSpace(QQ,1,-dim[b])
           M.append(MS())
           for i in range(-dim[b]):
                   M[b][0,i] = linearSolution[cnt]
                   cnt=cnt+1
       if dim[b] == 0:
           MS = MatrixSpace(QQ,0,0)
           M.append(MS())
 
   return M

def matrixToLinear(blocks,dim,M):
   variables = 0
   for b in range(blocks):
      variables += (dim[b]*(dim[b]+1))/2
   L=vector(QQ,variables)
   cnt = 0
   for b in range(blocks):
       for i in range(dim[b]):
           for j in range(i,dim[b]):
               L[cnt] = M[b][j,i]
               cnt=cnt+1
   return L

############################################################################ end of utilities

class FAProblem:
    """Contains problem and solution"""


    def __init__(self,filename):
        """Initialization"""

        self.data = []
        self.problem=None
        self.solnMatrix=None

        self.tightgraphs=[]
        self.tightgraphsIDS=[]
        self.extraTight=[]
        
        self.filename=filename
        self.problem=read_problem_file(self.filename)
        self.solution=read_solution_file(self.filename+".result",self.problem)
        self.solnMatrix = self.solution['Primal']


    def test_ranks(self):
        """Testing ranks of matrices in the solution"""
        block_sizes=self.problem['block_sizes']
        nextBlock = 0
        for idA in range(len(self.solnMatrix)):
            # These are slack variables - do not care about these guys
            if -block_sizes[idA] == self.problem['num_constraints']+1:
                continue
            A = self.solnMatrix[idA]
            # Do not bother with matrices that are not squares
            # they do not need any special attention
            if not A.is_square():
                continue
            # Now test rank of A
            print "Block",idA,"of size",A.ncols(),"has rank",A.rank()



    def test_zero_eigenvalues(self):
        """Prints smallest eigenvalue for every block of solution."""
        global problem
        global solutionMatrix
        block_sizes=self.problem['block_sizes']
        nextBlock = 0
        for idA in range(len(self.solnMatrix)):
            # These are slack variables - do not care about these guys
            if -block_sizes[idA] == self.problem['num_constraints']+1:
                continue
            A = self.solnMatrix[idA]
            # Do not bother with matrices that are not squares
            # they do not need any special attention
            if not A.is_square():
                continue
            # Now we try to find all eigenvalues that are small
            zero_eigenvalues=[]
            smallest_ev = -1
            for ev in A.eigenvectors_right():
                if (smallest_ev == -1 or abs(ev[0]) < smallest_ev):
                    smallest_ev = abs(ev[0])
                if (abs(ev[0]) < 0.0001):
                    zero_eigenvalues.append(abs(ev[0]))
            print "Block",idA,"has smallest eigenvalue",smallest_ev

    # Prints ten smallest eigenvalues in block idA
    # This is usefull 
    def test_block(self,idA):
        block_sizes=self.problem['block_sizes']
        A = self.solnMatrix[idA]
        if not A.is_square():
            print "Not a square matrix"
            return
        X = sorted([ real_part(ev[0]) for ev in A.eigenvectors_right()])
        print X[0:min(len(X),10)]
        return X







    # Try to identify tight graphs
    # This is a sanity check - should not be changing when removing types
    # returns lists that contain 3 elemets:
    # 1... value of slack variable.  Slack = 0 means tight graph
    # 2... value in dual. Dual >= means tight graph (appears in extremal construction)
    # 3... ID in order of the graph
    # can be sorted using
    #  sa=sorted(x, key = lambda y: y[0])
    #  sa=sorted(x, key = lambda y: y[1])
    #
    def guess_tight_graphs(self,threshold = 0.000001):
        block_sizes=self.problem['block_sizes']
        self.tightgraphs=[]
        self.tightgraphsIDS=[]
        tightlist=[]
        # test if zero in primal vector
        for idA in range(len(self.solnMatrix)):
            # These are slack variables - do not care about these guys
            if -block_sizes[idA] == self.problem['num_constraints']+1:
                A = self.solnMatrix[idA] 
                # Notice that solnMatrix[idA][0,0] contains the extra variable for
                # objective function. So the first slack is at solnMatrix[idA][0,1]
                for i in range(self.problem['num_constraints']):
                    self.tightlist.append([A[0,i+1],self.solution['Y'][i],i])
                    #tightlist.append([A[0,i+1],solution['Y'][i],density_in_blowup(i),i])
                    if A[0,i+1] < threshold:
                        print 'tight',i,tightlist[i]
                        self.tightgraphs.append(1)
                        self.tightgraphsIDS.append(i)
                    else:
                        self.tightgraphs.append(0)
                        print 'loose',i,tightlist[i]
            print "idA=",idA
            break
        return tightlist
        # test if non-zero in dual vector. It is slightly unprecise
        # as it only works for complementary slackness
        '''
        for i in range(len(solution['Y'])):
            if solution['Y'][i] > 0.0000001:
                tightgraphs.append(1)
                tightgraphsIDS.append(i)
            else:
                tightgraphs.append(0)
        '''



    # Copies problem and does a base change based on zero vectors.
    # Zero vectors are vectors we believe are zero eigenvectors of
    # blocks in the problem matrix. If we find them all, it would
    # be possible that CSDP returns to us a matrix that is positive
    # definite instead of positive semidefinite and it becomes much
    # easier to round.
    #
    def change_base(self,zerovectors):
        global problem
        global solutionMatrix
        newproblem = deepcopy(self.problem);
        block_sizes=self.problem['block_sizes']
        nextBlock = 0
        for idA in range(len(self.solnMatrix)):
            if -block_sizes[idA] == self.problem['num_constraints']+1:
                continue
            A = self.solnMatrix[idA]
            # Do not bother with matrices that are not squares
            # they do not need any special attention
            # Or if they are just constants...
            if not A.is_square() or (1 == A.nrows() == A.ncols()):
                #newproblem['blocks'] = newproblem['blocks']+[problem['blocks'][idA]]
                continue

            if len(zerovectors) < nextBlock:
                print "Not enought zero vectors"
                return
            if len(zerovectors[nextBlock]) > 0:
                print "Changing base of block",idA,"using",len(zerovectors[nextBlock]),"eigenvector(s) for zero"
                V = Matrix(zerovectors[nextBlock])
                # Orthogonal complement of our eigenvector
                RestBase = V.right_kernel_matrix()
                #print "Base orthogonal to zerovector: T="
                #print RestBase
                #print RestBase*A*RestBase.transpose()
        
                for idX in range(len(newproblem['blocks'][idA])):
                    newproblem['blocks'][idA][idX] = RestBase*newproblem['blocks'][idA][idX]*RestBase.transpose()
                newproblem['block_sizes'][idA] -= len(zerovectors[nextBlock])
       

                #normZ = vector(zerovector).norm()
                #normalZ = [ float(y/normZ) for y in zerovector ]
                #print "Normalized eigenvector for zero:",normalZ
                #T = Matrix([[ float(y) for y in z/z.norm()] for z in RestBase.rows()]+[normalZ])
                #T = Matrix(RestBase.rows()+[zerovector])
                #print "Matrix T*A*T^T="
                #print T*A*T.transpose()
                #X = Matrix(RestBase.rows()+[zerovector]).transpose()
                #print X
                #print X.inverse()*A*X
            else:
                print "Block",idA,"has no zero eigenvalues."
            nextBlock+=1
            #print "Done with",idA
        return newproblem




    def perform_rounding(self,optimal_value,denominator=1000000000):
        
        success=True
        
        # Try to identify tight graphs
        tightgraphs = []
        tightgraphsIDS = []
        for i in range(len(self.solution['Y'])):
            if self.solution['Y'][i] > 0.0000001 or i in self.extraTight:
                tightgraphs.append(1)
                tightgraphsIDS.append(i)
            else:
                #print 'Not tight:',solution_alt['Y'][i]
                tightgraphs.append(0)


        numtight = len(tightgraphsIDS)
        constraints = numtight

        print "Guessed",numtight,"tight graphs"


        blocks = len(self.problem['blocks']) 

        dim = []  # may be negative 
        variables = 0


        for i in range(blocks):
            blocksize=self.problem['block_sizes'][i]
            # Slack variables
            if (blocksize == -self.problem['num_constraints']-1):
                dim.append(0)
                continue
            if (blocksize > 0):
                dim.append(self.problem['block_sizes'][i])
                variables += dim[i]*(dim[i]+1)/2
            else:
                dim.append(self.problem['block_sizes'][i])
                variables += -dim[i]



        indexes = [] # matrix containing indeces for variables
        cnt = 0
        for b in range(blocks):
            #print dim[b]
            if dim[b] > 0:
                MS = MatrixSpace(QQ,dim[b])
                indexes.append(MS())
                for i in range(dim[b]):
                    for j in range(i,dim[b]):
                        indexes[b][j,i] = indexes[b][i,j] = cnt
                        cnt = cnt+1
            if dim[b] < 0:
                MS = MatrixSpace(QQ,1,-dim[b])
                indexes.append(MS())
                for i in range(-dim[b]):
                    indexes[b][0,i] = cnt
                    cnt = cnt+1
            if dim[b] == 0:
                indexes.append(-1)


        # Linearize the solution
        print "Making solution linear in",variables,"variables"
        solutionX = vector(QQ,variables)
        print solutionX
        cnt = 0
        for b in range(blocks):
            if dim[b] > 0:
                for i in range(dim[b]):
                    for j in range(i,dim[b]):
                        solutionX[cnt] = self.solution['Primal'][b][i][j]
                        cnt=cnt+1
            if dim[b] < 0:
                for i in range(-dim[b]):
                        solutionX[cnt] = self.solution['Primal'][b][0][i]
                        cnt=cnt+1

        # ROUND
        print "Testing QQ solution"
        densityVectorX = checkSolution(self.problem, solutionX)
        print 'Min of denisty vector for original solution is', float(min(densityVectorX)),'and we hope for',optimal_value



        print "Constructing matrix of constriants for tight graphs"
        MS = MatrixSpace(QQ,constraints,variables)
        A = MS()  # zero matrix
        Y = vector(QQ,constraints)

        # constraints for tight graphs:
        # Note that the first g=0 is the objective function
        next_constraint = 0
        for g in range(self.problem['num_constraints']):
            if tightgraphs[g] == 1:
                for b in range(blocks):
                    if dim[b] > 0:
                        for i in range(dim[b]):
                            for j in range(dim[b]):
                                if i == j:
                                    A[next_constraint, indexes[b][i,j]] = self.problem['blocks'][b][g+1][i][j]
                                else:
                                    A[next_constraint, indexes[b][i,j]] = 2*self.problem['blocks'][b][g+1][i][j]

                    if dim[b] < 0:
                        for i in range(-dim[b]):
                            A[next_constraint, indexes[b][0,i]] = self.problem['blocks'][b][g+1][0][i]
                Y[next_constraint] = self.problem['constants'][g]-optimal_value  # 1 is the optimal solution
                next_constraint += 1


        # Now matrix A should not be modified any more
        # it will lead to A.rank() being fast once computed once
        A.set_immutable()
        print "Rank of A is", A.rank()



        print "Creating and testing entry-wise rounded solution"
        # Rounding the solution:
        #denominator=1000000000000000000
        #denominator=1000000000
        solutionRounded = vector(QQ,len(solutionX))
        for i in range(len(solutionX)):
            solutionRounded[i] = floor(solutionX[i]*denominator)
            solutionRounded[i] /= denominator


        densityVectorR = checkSolution(self.problem, solutionRounded) 
        print 'Min of denisty just rounded vector is', float(min(densityVectorR)),'and we hope for',optimal_value
        #print 'Max of denisty vector is', float(max(densityVector))
        #print [ float(z) for z in densityVector ]


        # Finding linearly independent columns
        pivots=A.pivots()
        print "Pivots are",pivots

        MS = MatrixSpace(QQ,constraints,len(pivots))
        X = MS()
        use_presolved = []

        for p in range(len(pivots)):
            X[:,p] = A[:,pivots[p]]

        YforX = copy(Y)
        for i in range(variables):
            if not i in pivots:
                for j in range(constraints):
                    YforX[j] -= A[j,i]*solutionRounded[i]

        print "Solving for pivot variables"
        not_presolved=X.solve_right(YforX)


        # Round all that are not presolved
        print "Differences in solved variables compared to rounded values (should be small)"
        for p in range(len(pivots)):
            print "Difference:",float(solutionRounded[pivots[p]]-not_presolved[p])
            solutionRounded[pivots[p]] = not_presolved[p]


        print "Checking the FINAL solution"
        densityVector = checkSolution(self.problem, solutionRounded) 
        print 'Min of denisty vector is', float(min(densityVector)),'and we hope for',optimal_value
        #print 'Max of denisty vector is', float(max(densityVector))

        # These are the variables that violate the constriants
        violators=[]
        for i in range(len(densityVector)):
            if densityVector[i] < optimal_value:
                violators.append(i)

        if len(violators) == 0:
            print "Rounding successfull!!"
        else:
            print "Violated constriants:",violators
            success=False



        print "Testing if the rounded solution is positive definite"
        M=linearToMatrix(blocks,dim,solutionRounded)
        for i in range(len(M)):
            B = M[i]
            if B.is_square() and B.ncols() >= 1:
                if B.is_positive_definite():
                    print "Block",i,"is positive definite, min eigval is",float(min(B.eigenvalues()))
                else:
                    print "Block",i,"IS NOT POSITIVE DEFINITE",float(min(B.eigenvalues()))
                    success=False
            else:
                for j in range(B.ncols()):
                    if B[0,j] < 0:
                        print "Block",i,"IS NOT POSITIVE DEFINITE"
                        success=False

        if success:
            print "Congratulations! Rounding succeeded"
        else:
            print "Rounding failed"


        self.solutionRounded = solutionRounded
        self.solutionRoundedM = M






#newproblem = deepcopy(problem);




unlabeled_flags=[]
# requires one flag per line. Nice in order.
def load_unlabeled_flags(fname):
    global unlabeled_flags
    fname="F_edges2blind__n8_unlabeled.txt"
    unlabeled_flags=[]
    with open(fname) as f:
        content = f.readlines()
        # remove whitespace characters like `\n` at the end of each line
        content = [x.strip() for x in content]
        for s in content:
            if s[0] == '8':
                unlabeled_flags.append(s)
    return unlabeled_flags

#blow_up='5 0 2 1 1 2  1 1 2  1 2  2'


# Computes density of a graph in blow-up. This is somewhat slow
# as it is using external flag program
def density_in_blowup(fID):
    global unlabeled_flags
    global blow_up
    data=subprocess.Popen("./a.out -FinbupH '{}'  '{}' 2>/dev/null".format(unlabeled_flags[fID], blow_up),shell=True,stdout=subprocess.PIPE).communicate()[0].strip()
    return float(data[9:])





# zero vectors should correspond to the zero eigenvectors
# we do a test by multiplying the corresponding vectors and matrices
# and see what is happening.
def test_zerovectors(zerovectors):
    global problem
    global solutionMatrix
    block_sizes=problem['block_sizes']
    nextBlock = 0
    for idA in range(len(solnMatrix)):
        if -block_sizes[idA] == problem['num_constraints']+1:
            continue
        A = solnMatrix[idA]
        # Do not bother with matrices that are not squares
        # they do not need any special attention
        if not A.is_square():
            continue

        if len(zerovectors) <= nextBlock:
            print "Not enought zero vectors, missing for block",nextBlock
            return
        if len(zerovectors[nextBlock]) > 0:
            for v in range(len(zerovectors[nextBlock])):
                Av = A*vector(zerovectors[nextBlock][v])
                print 'Block',idA,"provided zero vector id",v,"gives when mutliplied with block  max value",max([max(Av),-min(Av)])
        #else:
        #    print "Block",idA,"has no zero eigenvalues."
        nextBlock+=1



def rounding_help():
    print "This program helps create rounding"
